/*
  Twin relay 
  Copyright (c) 2013 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
  Release Notes:


*/  

#include "Arduino.h"
#include "bv4601_I.h"

// =============================================================================
// SV3B Type serial control
// =============================================================================
// =============================================================================
// *****************************************************************************
// constructor is just a connection to NewSoftSerial
// *****************************************************************************
RELAY::RELAY(char i2cadr)
{
  _i2adr=i2cadr;  
  Wire.begin(); // join i2c bus
}

// *****************************************************************************
// turns relay 'n' on in a give time, n is eithe 0 or not 0
// 0 coresponds to relay 'a'
// *****************************************************************************
void RELAY::on(char n, int timeOn)
{
    Wire.beginTransmission(_i2adr);
    if(n) Wire.write(2); // select relay as command
    else Wire.write(1);
    Wire.write(1); // on command
    Wire.write(timeOn / 256); // time high
    Wire.write(timeOn % 256); // time low
    Wire.endTransmission();
}

// *****************************************************************************
// turns relay 'n' off in a give time, n is eithe 0 or not 0
// 0 coresponds to relay 'a'
// *****************************************************************************
void RELAY::off(char n, int timeOn)
{
    Wire.beginTransmission(_i2adr);
    if(n) Wire.write(2); // select relay as command
    else Wire.write(1);
    Wire.write(0); // off command
    Wire.write(timeOn / 256); // time high
    Wire.write(timeOn % 256); // time low
    Wire.endTransmission();
}

// *****************************************************************************
// turns all relays off
// 0 coresponds to relay 'a'
// *****************************************************************************
void RELAY::alloff()
{
    Wire.beginTransmission(_i2adr);
    Wire.write(6); // all off
    Wire.endTransmission();
}

// *****************************************************************************
// sets voltage reference 1 to 4
// *****************************************************************************
void RELAY::vref(char v)
{
    Wire.beginTransmission(_i2adr);
    Wire.write(7); // vref
    if(v < 1) v = 1;
    if(v > 4) v = 4;
    Wire.write(v);
    Wire.endTransmission();
}
 
// *****************************************************************************
// sets digital output to be either out or in, n=0 is out n!=0 is in
// *****************************************************************************
void RELAY::digital(char n)
{
    Wire.beginTransmission(_i2adr);
    Wire.write(8); // set digital
    if(n) Wire.write(1);
    else Wire.write(0);
    Wire.endTransmission();
}

// =============================================================================
// Input
// =============================================================================
// =============================================================================

// **************************************************************
// gets a 16 bit value from the i2c bus
// **************************************************************
int RELAY::i2_16bit()
{
int rv;
  Wire.requestFrom(_i2adr, 2); // returns 2 bytes
  rv=Wire.read()*256; // high byte
  rv+=Wire.read(); // low byte
  return rv;
}

// *****************************************************************************
// returns value of current relay timer 0 is relay a, not 0 is relay b
// *****************************************************************************
int RELAY::gettimer(char relay)
{
    Wire.beginTransmission(_i2adr);
    Wire.write(4); // set digital
    if(relay) Wire.write(2);
    else Wire.write(1);
    Wire.endTransmission();
    return i2_16bit();
}

// *****************************************************************************
// gets adc value from adc channel 1,2 or 3
// *****************************************************************************
int RELAY::adc(char n)
{
    if(n < 1) n = 1;
    if(n > 3) n = 3;
    Wire.beginTransmission(_i2adr);
    Wire.write(5); // get adc
    Wire.write(n);
    Wire.endTransmission();
    return i2_16bit();
}

